@@ -1,5 +1,5 @@
 #!/bin/bash
-DIR=web/xhotkeys
+DIR=webpage/xhotkeys
 FILES="index-template.html
 howtouse-template.html
 contact-template.html
