@@ -55,61 +55,23 @@ TARGET_TYPE_URI_LIST = 80
 LEVELS_STRING = "error", "info", "debug"
 ERROR, INFO, DEBUG = range(3)
 
-### KeySymbol to (XMask/strMask) conversion
-x, xk, xkb = Xlib.X, Xlib.XK, Xlib.keysymdef.xkb
-
-keysym_to_mask = {
-	xk.XK_Shift_L: (x.ShiftMask, "shift"), 
-	xk.XK_Shift_R: (x.ShiftMask, "shift"),
-	xk.XK_Control_L: (x.ControlMask, "control"),
-	xk.XK_Control_R: (x.ControlMask, "control"),
-	xk.XK_Alt_L: (x.Mod1Mask, "alt"),
-	xk.XK_Alt_R: (x.Mod1Mask, "alt"),
-	xk.XK_Super_L: (x.Mod4Mask, "winkey"),
-	xkb.XK_ISO_Level3_Shift: (x.Mod5Mask, "altgr")
-}
+x = Xlib.X
+MODIFIERS = [("shift", x.ShiftMask), ("lock", x.LockMask), ("control", x.ControlMask), \
+	("alt", x.Mod1Mask), ("mod2", x.Mod2Mask), ("mod3", x.Mod3Mask), \
+	("winkey", x.Mod4Mask), ("altgr", x.Mod5Mask)]
 
 ########################################
 ## From Peter Norvig's Infrequently Answered Questions
 def create_dict(**dict): 
 	return dict
 
-########################################
-def keycodes_to_mask(display, keycodes):
-	"""Walk keycodes elements and returns only modifiers masks"""
-	mask = 0
-	for keycode in keycodes: 
-		for keysym, tmask in keysym_to_mask.items():
-			if keycode == display.keysym_to_keycode(keysym):
-				mask = mask | tmask[0]
-	return mask
-
-########################################
-def string_to_mask(str_modifiers):
-	"""Return modifier mask from its name: "shift", "alt", ..."""
-	mask = 0
-	for str_modifier in str_modifiers:
-		for keysym, tmask in keysym_to_mask.items():
-			if tmask[1] == str_modifier.lower():
-				mask = mask | keysym_to_mask[keysym][0]
-				break
-	return mask
-
-########################################
-def get_total_mask():
-	"""Return OR-mask of all modifiers"""
-	mask = 0
-	for xmask, name in keysym_to_mask.values():
-		mask = mask | xmask
-	return mask
-	
 #######################################################
 def debug(text, verbose_level, level, exit=None):
 	"""Give some debug messages.
 	
 	verbose_level -- verbose level for this message: DEBUG, INFO, ERROR
-	level -- log-level: DEBUG, INFO, ERROR
-	exit -- exit with that return value"""
+	level -- current log-level: DEBUG, INFO, ERROR
+	exit -- if defined, finish with this exit code"""
 	if level <= verbose_level:
 		text = "%s: %s" %(LEVELS_STRING[level], text)
 		sys.stderr.write(text + "\n")
@@ -141,6 +103,14 @@ class XhotkeysServer(threading.Thread):
 		self.stop_flag = False
 		threading.Thread.__init__(self)
 
+	########################################
+	def string_to_mask(self, strmods):
+		"""Return modifier mask from its name: "shift", "alt", ..."""
+		mask = 0
+		for strmod in strmods:
+			mask |= dict(MODIFIERS)[strmod.lower()]
+		return mask
+
 	########################################
 	def string_to_keycode(self, strcode):
 		if len(strcode) > 2 and strcode[0] == "@" and strcode[-1] == "@":
@@ -148,7 +118,7 @@ class XhotkeysServer(threading.Thread):
 		if strcode.find(MOUSE_STRING) == 0:
 			return "button", int(strcode[-1])
 		for key in inspect.getmembers(Xlib.XK):
-			if key[0].find("XK_") != 0 or strcode != key[0].replace("XK_", ""): continue
+			if key[0].find("XK_") != 0 or strcode.lower() != key[0].replace("XK_", "").lower(): continue
 			return "key", self.display.keysym_to_keycode(key[1])
 
 	##################################################
@@ -208,7 +178,7 @@ class XhotkeysServer(threading.Thread):
 			type = mask = keycode = 0
 			if hotkey.lower() != DISABLED_STRING.lower():
 				modifiers = re.findall("<(\w+)>", hotkey)
-				mask = string_to_mask(modifiers)
+				mask = self.string_to_mask(modifiers)
 				try: key = re.findall("(@?\w+@?)$", hotkey)[0]
 				except: self.debug("error parsing hotkey: %s" %hotkey, ERROR); continue
 				try: type, keycode = self.string_to_keycode(key)
@@ -262,7 +232,7 @@ class XhotkeysServer(threading.Thread):
 		while not self.stop_flag:
 			while self.display.pending_events():
 				event = self.display.next_event()
-				mask = event.state & get_total_mask()
+				mask = event.state
 				self.debug(event)
 				if event.type == Xlib.X.KeyPress:
 					keycode = event.detail					
@@ -292,6 +262,17 @@ class Xhotkeys:
 		self.display = Xlib.display.Display()
 		self.root = self.display.screen().root
 		self.hkserver = None
+		self.create_keysym_mask(MODIFIERS)
+
+	####################################
+	def create_keysym_mask(self, modifiers):
+		self.keysym_to_mask = {}
+		self.keysym_to_maskname = {}
+		for maskindex, mmlist in enumerate(self.display.get_modifier_mapping()):
+			for ks in [self.display.keycode_to_keysym(mm, index) for mm in mmlist if mm != 0 for index in range(0,4)]:
+				if ks != Xlib.X.NoSymbol: 
+					self.keysym_to_maskname[ks], self.keysym_to_mask[ks] = modifiers[maskindex]
+		self.debug("keysym_to_maskname table: %s" %str(self.keysym_to_maskname))
 
 	####################################
 	def signal_handler(self, signum, frame):
@@ -323,16 +304,31 @@ class Xhotkeys:
 
 	########################################
 	def is_keycode_modifier(self, keycode):
-		return bool([ks for ks in keysym_to_mask if keycode == self.display.keysym_to_keycode(ks)])
+		return bool([ks for ks in self.keysym_to_mask if keycode == self.display.keysym_to_keycode(ks)])
 
+	########################################
+	def keycodes_to_mask(self, keycodes):
+		"""Walk keycodes elements and returns only modifiers masks"""
+		mask = 0
+		for keycode in keycodes:
+			for keysym in self.keysym_to_mask:
+				if keycode in self.display.keysym_to_keycodes(keysym):
+					mask |= self.keysym_to_mask[keysym]
+		return mask
+		
 	########################################
 	def keycodes_to_string(self, keycodes, default_string=DISABLED_STRING):
 		if not keycodes: return default_string			
 		keycodes_to_modifier = {}
-		for keysym, tmask in keysym_to_mask.items():
-			modifier = tmask[1]
-			keycodes_to_modifier[self.display.keysym_to_keycode(keysym)] = modifier
-
+		for keycode in keycodes:
+			for keysym in self.keysym_to_mask:
+				if keycode in [x[0] for x in self.display.keysym_to_keycodes(keysym)]:
+					string = self.keysym_to_maskname[keysym]
+					if string.lower() == "altgr": name = "AltGr"
+					elif string.lower() == "winkey": name = "WinKey"
+					else: name = string.title()
+					print name
+					keycodes_to_modifier[keycode] = name
 		strmod = strkey = ""
 		current_modifiers = []
 		for keycode in keycodes:
@@ -345,7 +341,7 @@ class Xhotkeys:
 			keysym = self.display.keycode_to_keysym(keycode, 0)
 			for key in inspect.getmembers(Xlib.XK):
 				if len(key) != 2 or key[0].find("XK_") != 0 or key[1] != keysym: continue
-				strkey = key[0].replace("XK_", "")
+				strkey = key[0].replace("XK_", "").title()
 				break
 			else: strkey = "@%d@" %keycode
 			break
@@ -445,7 +441,7 @@ class Xhotkeys:
 
 	#######################################
 	def set_hotkey_text(self, text):
-		self.widgets["hotkey"].set_text(text.title())
+		self.widgets["hotkey"].set_text(text)
 		
 	#######################################
 	def error_message_response(self, widget, retval):
@@ -475,7 +471,7 @@ class Xhotkeys:
 			self.set_hotkey_text(DISABLED_STRING)
 			return
 		if not keycodes: self.set_hotkey_text(self.current_comb["hotkey"]); return			
-		mask = keycodes_to_mask(self.display, keycodes)
+		mask = self.keycodes_to_mask(keycodes)
 		keycode = keycodes[-1]
 		newcomb = create_dict(mask=mask, keycode=keycode, hotkey=hotkey)
 
@@ -524,7 +520,7 @@ class Xhotkeys:
 		statfile = "/proc/%d/stat" % pid 
 		try: fd = open(statfile)
 		except IOError: self.debug("cannot read process status (%s)" %statfile); return
-		if fd.read().split()[1] != "(%s)" %NAME: 
+		if fd.readline().split()[1].find("(%s" %NAME) != 0: 
 			self.debug("pidfile found but not a %s daemon, so deleting it" %NAME)
 			try: os.unlink(self.pidfile)
 			except: self.debug("error deleting pidfile" %self.pidfile, ERROR)
@@ -725,7 +721,7 @@ class Xhotkeys:
 
 	#########################################
 	def on_hotkey_button_press_event(self, widget, event):
-		if not keycodes_to_mask(self.display, self.keycodes_pressed): return
+		if not self.keycodes_to_mask(self.keycodes_pressed): return
 		string = self.keycodes_to_string(self.keycodes_pressed)
 		self.hotkey_end(self.keycodes_pressed, string + MOUSE_STRING + str(event.button))
 		widget.stop_emission("button_press_event")
@@ -891,6 +887,7 @@ class Xhotkeys:
 	def save_settings(self):
 		try: fd = open(self.settingsfile, "w")
 		except: self.debug("settings could not be opened for writing: %s" %self.settingsfile); return
+		fd.write("# automatically saved settings xhotkeys\n")
 		x, y = self.widgets["mainwindow"].get_position()
 		width, height = self.widgets["mainwindow"].get_size()
 		fd.write(SETTINGS_WINDOW % (x, y, width, height) + "\n")
@@ -978,7 +975,7 @@ Bind keys and mouse combinations to commands in the X-Window System"""
 	home, user = os.getenv("HOME"), os.getenv("USER")
 	default_cfile = os.path.join(home, ".%s" %NAME)
 	default_pidfile = os.path.join(home, ".%s.pid-%s" %(NAME, user))
-	default_settings = os.path.join(home, ".%s.settings" %NAME)
+	default_settings = os.path.join(home, ".%s.settings-%s" %(NAME, user))
 	parser = optparse.OptionParser(usage)
 	parser.add_option('-v', '--verbose', default=0, dest='verbose_level', action="count", help='Increase verbose level')
 	parser.add_option('-c', '--config', dest='config', default=False, action='store_true', help='Open a GTK+ interface configurator')
